%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Ce script montre l'utilisation de deux fonctions Matlab 
%%%% pour générer le diagramme de Voronoï d'un environnement Q  
%%%% à partir de n generateurs (points 2D répartis de façon aléatoire)
%%%%
%%%% 1) La fonction "mpt_voronoi(.,.)" utilise la programmation linéaire
%%%% multi-paramètrique via le Multi-Parametric Toolbox (MPT)
%%%% http://control.ee.ethz.ch/~mpt/downloads/
%%%% (exécuter le script "install_mpt3.m"),
%%%% pour déterminer les régions V1,...,Vn du diagramme de Voronoï
%%%%
%%%% 2) La fonction "voronoi(.,.)" construit aussi le diagramme de Voronoï,
%%%% mais la frontière de l'environnement Q à partitionner ne peut pas 
%%%% être specifiée comme dans "mpt_voronoi(.,.)". "voronoi(.,.)" est 
%%%% une fonction standard de Matlab 
%%%%                                                          
%%%% Author: Fabio Morbidi                                    
%%%% Décembre 2018

close all
clear 
clc

n = 40; %% Nombre de generateurs du diagramme de Voronoï 

mu = [0.9 0.3];
Sigma = [0.21 0;0 0.21]; 
R = chol(Sigma);

%%% Les générateurs sont des points aléatoires tirés d'une pdf gaussienne
%%% bivariée avec moyenne 'mu' et matrice de covariance 'Sigma'

points = repmat(mu,n,1) + randn(n,2)*R; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Utilisation de la fonction "mpt_voronoi" du Multi-Parametric Toolbox %%%

%%% Définition de l'environnement Q (polyèdre) que nous voulons  
%%% partitionner: ici, une représentation par sommets (coordonnées x et y) 
%%% est utilisée

Q = Polyhedron([-2, -0.5; 2, -2; 4, 1.2; -3, 1; 0, 2.5; 4 -0.5]);

figure(1)
[V, P] = mpt_voronoi(points','bound', Q); %% V contient les n regions de Voronoï V1,...,Vn 
plot(V)
hold on
%% Affichage des générateurs
for i=1:n
    plot(points(i,1),points(i,2),'k.')
    text(points(i,1),points(i,2),['  ',num2str(i)],...
    'VerticalAlignment','middle','FontSize',10)    
end
title('Diagramme de Voronoï généré par la fonction "mpt\_voronoi"')
axis equal
grid off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Utilisation de la fonction standard de Matlab "voronoi" %%%

figure(2)
voronoi(points(:,1),points(:,2))
xlabel('x_1')
ylabel('x_2')
axis equal
title('Diagramme de Voronoï généré par la fonction "voronoi"')
